@@ -58,6 +58,7 @@ void xsane_progress_cancel(GtkWidget *widget, gpointer data);
 void xsane_progress_new(char *bar_text, char *info, GtkSignalFunc callback, int *cancel_data_pointer);
 void xsane_progress_update(gfloat newval);
 void xsane_progress_clear();
+void xsane_progress_bar_set_fraction(GtkProgressBar *progress_bar, gdouble fraction);
 GtkWidget *xsane_vendor_pixmap_new(GdkWindow *window, GtkWidget *parent);
 GtkWidget *xsane_toggle_button_new_with_pixmap(GdkWindow *window, GtkWidget *parent, const char *xpm_d[], const char *desc,
                                          int *state, void *xsane_toggle_button_callback);
@@ -661,6 +662,15 @@ gint xsane_authorization_callback(SANE_String_Const resource,
     hbox = gtk_hbox_new(TRUE, 10); /* x-space between buttons */
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 10);  /* y-space around buttons */
 
+#ifdef HAVE_GTK2
+    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#else
+    button = gtk_button_new_with_label(BUTTON_CANCEL);
+#endif
+    g_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(xsane_authorization_button_callback), (void *) -1);
+    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 10); /* x-space around cancel-button */
+    gtk_widget_show(button);
+
 #ifdef HAVE_GTK2
     button = gtk_button_new_from_stock(GTK_STOCK_OK);
 #else
@@ -672,15 +682,6 @@ gint xsane_authorization_callback(SANE_String_Const resource,
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
 
-#ifdef HAVE_GTK2
-    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-#else
-    button = gtk_button_new_with_label(BUTTON_CANCEL);
-#endif
-    g_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(xsane_authorization_button_callback), (void *) -1);
-    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 10); /* x-space around cancel-button */
-    gtk_widget_show(button);
-
     gtk_widget_show(hbox);
 
     gtk_widget_show(authorize_dialog);
@@ -734,6 +735,49 @@ gint xsane_authorization_callback(SANE_String_Const resource,
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+void xsane_progress_bar_set_fraction(GtkProgressBar *progress_bar, gdouble fraction)
+{
+   if (fraction < 0.0)
+   { 
+     fraction = 0.0; 
+   }
+
+   if (fraction > 1.0)
+   { 
+     fraction = 1.0; 
+   }
+
+#ifdef HAVE_GTK2
+  if ((fraction - gtk_progress_bar_get_fraction(progress_bar) > XSANE_PROGRESS_BAR_MIN_DELTA_PERCENT) || (fraction == 0.0) || (fraction > 0.99))
+  {
+    gtk_progress_bar_set_fraction(progress_bar, fraction);
+#if 0
+    /* this produces jumping scrollbars when we use it instead of the gtk_main_iteration loop */
+    if (GTK_WIDGET_DRAWABLE(progress_bar))
+    {
+      gdk_window_process_updates(progress_bar->progress.widget.window, TRUE);
+    }
+#endif
+    while (gtk_events_pending())
+    {
+      gtk_main_iteration();
+    }
+  }
+#else
+  if ((fraction - gtk_progress_get_current_percentage(&progress_bar->progress) > XSANE_PROGRESS_BAR_MIN_DELTA_PERCENT) || (fraction == 0.0) || (fraction > 0.99))
+  {
+    gtk_progress_bar_update(progress_bar, fraction);
+
+    while (gtk_events_pending())
+    {
+      gtk_main_iteration();
+    }
+  }
+#endif
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 void xsane_progress_cancel(GtkWidget *widget, gpointer data)
 {
  void *cancel_data_pointer;
@@ -1015,7 +1059,7 @@ void xsane_option_menu_new(GtkWidget *parent, char *str_list[], const char *val,
 
   option_menu = gtk_option_menu_new();
   xsane_back_gtk_set_tooltip(xsane.tooltips, option_menu, desc);
-  gtk_box_pack_end(GTK_BOX(parent), option_menu, FALSE, FALSE, 2);
+  gtk_box_pack_end(GTK_BOX(parent), option_menu, TRUE, TRUE, 5);
   gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
   gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), xsane_option_menu_lookup(menu_items, val));
 
@@ -1043,7 +1087,7 @@ void xsane_option_menu_new_with_pixmap(GdkWindow *window, GtkBox *parent, const
   DBG(DBG_proc, "xsane_option_menu_new_with_pixmap\n");
 
   hbox = gtk_hbox_new(FALSE, 5);
-  gtk_box_pack_start(parent, hbox, FALSE, FALSE, 0);
+  gtk_box_pack_start(parent, hbox, FALSE, FALSE, 2);
 
   pixmap = gdk_pixmap_create_from_xpm_d(window, &mask, xsane.bg_trans, (gchar **) xpm_d);
   pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
@@ -1172,7 +1216,7 @@ void xsane_range_new_with_pixmap(GdkWindow *window, GtkBox *parent, const char *
   DBG(DBG_proc, "xsane_slider_new_with_pixmap\n");
 
   hbox = gtk_hbox_new(FALSE, 5);
-  gtk_box_pack_start(parent, hbox, FALSE, FALSE, 1);
+  gtk_box_pack_start(parent, hbox, FALSE, FALSE, 2);
 
   pixmap = gdk_pixmap_create_from_xpm_d(window, &mask, xsane.bg_trans, (gchar **) xpm_d);
   pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
@@ -1262,6 +1306,7 @@ static void xsane_browse_filename_callback(GtkWidget *widget, gpointer data)
 {
  char filename[PATH_MAX];
  char windowname[TEXTBUFSIZE];
+ int show_extra_widgets;
 
   DBG(DBG_proc, "xsane_browse_filename_callback\n");
 
@@ -1277,10 +1322,17 @@ static void xsane_browse_filename_callback(GtkWidget *widget, gpointer data)
     strcpy(filename, OUT_FILENAME);
   }
 
+  show_extra_widgets = XSANE_GET_FILENAME_SHOW_FILETYPE;
+  if (xsane.enable_color_management)
+  {
+    show_extra_widgets |= XSANE_GET_FILENAME_SHOW_CMS_FUNCTION;
+  }
+
+
   snprintf(windowname, sizeof(windowname), "%s %s %s", xsane.prog_name, WINDOW_OUTPUT_FILENAME, xsane.device_text);
 
   umask((mode_t) preferences.directory_umask); /* define new file permissions */    
-  xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, &preferences.filetype, TRUE, TRUE, FALSE, TRUE);
+  xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, &preferences.filetype, &preferences.cms_function, XSANE_FILE_CHOOSER_ACTION_SELECT_SAVE, show_extra_widgets, XSANE_FILE_FILTER_ALL | XSANE_FILE_FILTER_IMAGES, XSANE_FILE_FILTER_IMAGES);
   umask(XSANE_DEFAULT_UMASK); /* define new file permissions */    
 
   if (preferences.filename)
@@ -1294,6 +1346,13 @@ static void xsane_browse_filename_callback(GtkWidget *widget, gpointer data)
 
   xsane_back_gtk_filetype_menu_set_history(xsane.filetype_option_menu, preferences.filetype);
 
+#ifdef HAVE_LIBLCMS
+  if (xsane.enable_color_management)
+  {
+    gtk_option_menu_set_history(GTK_OPTION_MENU(xsane.cms_function_option_menu), preferences.cms_function);
+  }
+#endif
+
   /* correct length of filename counter if it is shorter than minimum length */
   xsane_update_counter_in_filename(&preferences.filename, FALSE, 0, preferences.filename_counter_len);
 
@@ -1412,13 +1471,12 @@ void xsane_outputfilename_new(GtkWidget *vbox)
 
   /* first line: disk icon, filename box */
 
-  hbox = gtk_hbox_new(FALSE, 2);
-  gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
+  hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
 
   button = xsane_button_new_with_pixmap(xsane.xsane_window->window, hbox, file_xpm, DESC_BROWSE_FILENAME,
                                         (GtkSignalFunc) xsane_browse_filename_callback, NULL);
-  gtk_widget_add_accelerator(button, "clicked", xsane.accelerator_group, GDK_B, GDK_CONTROL_MASK, DEF_GTK_ACCEL_LOCKED);
+  gtk_widget_add_accelerator(button, "clicked", xsane.accelerator_group, GDK_F, GDK_MOD1_MASK, DEF_GTK_ACCEL_LOCKED);
 
   text = gtk_entry_new();
   gtk_widget_set_size_request(text, 80, -1); /* set minimum size */
@@ -1427,7 +1485,7 @@ void xsane_outputfilename_new(GtkWidget *vbox)
   gtk_entry_set_text(GTK_ENTRY(text), (char *) preferences.filename);
   gtk_entry_set_position(GTK_ENTRY(text), strlen(preferences.filename)); /* set cursor to right position of filename */
 
-  gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 4);
+  gtk_box_pack_end(GTK_BOX(hbox), text, TRUE, TRUE, 5);
   g_signal_connect(GTK_OBJECT(text), "changed", (GtkSignalFunc) xsane_outputfilename_changed_callback, NULL);
 
   xsane.outputfilename_entry = text;
@@ -1438,9 +1496,8 @@ void xsane_outputfilename_new(GtkWidget *vbox)
 
   /* second line: Step, Type */
 
-  hbox = gtk_hbox_new(FALSE, 2);
-  gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
 
   /* filename counter step */
 
@@ -1452,7 +1509,7 @@ void xsane_outputfilename_new(GtkWidget *vbox)
  
   xsane_filename_counter_step_option_menu = gtk_option_menu_new();
   xsane_back_gtk_set_tooltip(xsane.tooltips, xsane_filename_counter_step_option_menu, DESC_FILENAME_COUNTER_STEP);
-  gtk_box_pack_start(GTK_BOX(hbox), xsane_filename_counter_step_option_menu, FALSE, FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(hbox), xsane_filename_counter_step_option_menu, FALSE, FALSE, 5);
   gtk_widget_show(xsane_filename_counter_step_option_menu);
   gtk_widget_show(hbox);
  
@@ -1477,7 +1534,7 @@ void xsane_outputfilename_new(GtkWidget *vbox)
   gtk_option_menu_set_history(GTK_OPTION_MENU(xsane_filename_counter_step_option_menu), select_item);
 
   xsane.filetype_option_menu = xsane_back_gtk_filetype_menu_new(preferences.filetype, (GtkSignalFunc) xsane_filetype_callback);
-  gtk_box_pack_end(GTK_BOX(hbox), xsane.filetype_option_menu, FALSE, FALSE, 2);
+  gtk_box_pack_end(GTK_BOX(hbox), xsane.filetype_option_menu, FALSE, FALSE, 5);
   gtk_widget_show(xsane.filetype_option_menu);
 
   xsane_label = gtk_label_new(TEXT_FILETYPE); /* opposite order because of box_pack_end */
@@ -1629,17 +1686,17 @@ void xsane_update_param(void *arg)
 
     if (xsane.param.format == SANE_FRAME_GRAY)
     {
-      xsane.xsane_colors = 1;
+      xsane.xsane_channels = 1;
     }
 #ifdef SUPPORT_RGBA
     else if (xsane.param.format == SANE_FRAME_RGBA)
     {
-      xsane.xsane_colors = 4;
+      xsane.xsane_channels = 4;
     }
 #endif
     else /* RGB */
     {
-      xsane.xsane_colors = 3;
+      xsane.xsane_channels = 3;
     }
   }
   else 
@@ -1785,6 +1842,7 @@ int xsane_identify_output_format(char *filename, char *filetype, char **ext)
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+#if 0
 void xsane_change_working_directory(void)
 {
  char filename[PATH_MAX];
@@ -1797,7 +1855,7 @@ void xsane_change_working_directory(void)
   sprintf(windowname, "%s %s %s", xsane.prog_name, WINDOW_CHANGE_WORKING_DIR, xsane.device_text);
   if (getcwd(filename, sizeof(filename)))
   {
-    xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, TRUE, FALSE, TRUE, FALSE);
+/*    xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, TRUE, FALSE, TRUE, FALSE); */
     if (chdir(filename))
     {
      char buf[TEXTBUFSIZE];
@@ -1819,6 +1877,7 @@ void xsane_change_working_directory(void)
 
   xsane_set_sensitivity(TRUE);
 }
+#endif
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
@@ -2019,11 +2078,6 @@ int xsane_display_eula(int ask_for_accept)
 
   if (ask_for_accept) /* show accept + not accept buttons */
   {
-    button = gtk_button_new_with_label(BUTTON_ACCEPT);
-    g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_eula_button_callback, (void *) 0 /* accept */);
-    gtk_container_add(GTK_CONTAINER(hbox), button);
-    gtk_widget_show(button);
-
     button = gtk_button_new_with_label(BUTTON_NOT_ACCEPT);
     gtk_widget_add_accelerator(button, "clicked", accelerator_group, GDK_Escape, 0, DEF_GTK_ACCEL_LOCKED);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
@@ -2031,6 +2085,11 @@ int xsane_display_eula(int ask_for_accept)
     gtk_container_add(GTK_CONTAINER(hbox), button);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
+
+    button = gtk_button_new_with_label(BUTTON_ACCEPT);
+    g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_eula_button_callback, (void *) 0 /* accept */);
+    gtk_container_add(GTK_CONTAINER(hbox), button);
+    gtk_widget_show(button);
   }
   else /* show close button */
   {
@@ -2396,16 +2455,6 @@ int xsane_front_gtk_getname_dialog(const char *dialog_title, const char *desc_te
   accelerator_group = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(getname_dialog), accelerator_group);
                                                                                                                                  
-#ifdef HAVE_GTK2
-  button = gtk_button_new_from_stock(GTK_STOCK_OK);
-#else
-  button = gtk_button_new_with_label(BUTTON_OK);
-#endif
-  g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_front_gtk_getname_button_callback, (void *) 1);
-  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-  gtk_widget_show(button);
-                                                                                                                                 
 #ifdef HAVE_GTK2
   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 #else
@@ -2416,6 +2465,16 @@ int xsane_front_gtk_getname_dialog(const char *dialog_title, const char *desc_te
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
   gtk_widget_show(button);
                                                                                                                                  
+#ifdef HAVE_GTK2
+  button = gtk_button_new_from_stock(GTK_STOCK_OK);
+#else
+  button = gtk_button_new_with_label(BUTTON_OK);
+#endif
+  g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_front_gtk_getname_button_callback, (void *) 1);
+  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+                                                                                                                                 
   xsane_front_gtk_getname_button = 0;
                                                                                                                                  
   while (xsane_front_gtk_getname_button == 0)
