@@ -505,6 +505,9 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
         status = sane_read(dev, (SANE_Byte *) buf16, sizeof(buf16), &len);
       }
 
+      DBG(DBG_info, "sane_read returned with status %s\n", XSANE_STRSTATUS(status));
+      DBG(DBG_info, "sane_read: len = %d\n", len);
+
 
       if (!xsane.scanning) /* scan may have been canceled while sane_read was executed */
       {
@@ -947,7 +950,7 @@ static int xsane_reduce_to_lineart()
  int abort = 0;
 
   /* open progressbar */
-  xsane_progress_new(PROGRESS_PACKING_DATA, PROGRESS_TRANSFERING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
+  xsane_progress_new(PROGRESS_PACKING_DATA, PROGRESS_TRANSFERRING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
   while (gtk_events_pending())
   {
     gtk_main_iteration();
@@ -1061,7 +1064,7 @@ void xsane_scan_done(SANE_Status status)
       image_info.image_width       = xsane.param.pixels_per_line;
       image_info.image_height      = pixel_height;
       image_info.depth             = xsane.depth;
-      image_info.colors            = xsane.xsane_colors;
+      image_info.channels          = xsane.xsane_channels;
 
       image_info.resolution_x      = xsane.resolution_x;
       image_info.resolution_y      = xsane.resolution_y;
@@ -1085,6 +1088,29 @@ void xsane_scan_done(SANE_Status status)
 
       image_info.reduce_to_lineart = xsane.expand_lineart_to_grayscale;
 
+      image_info.enable_color_management = xsane.enable_color_management;
+      image_info.cms_function            = preferences.cms_function;
+      image_info.cms_intent              = preferences.cms_intent;
+      image_info.cms_bpc                 = preferences.cms_bpc;
+
+      image_info.icm_profile[0] = 0; /* empty string */
+
+      if (image_info.channels == 1)
+      {
+        if (xsane.scanner_default_gray_icm_profile)
+        {
+          strncpy(image_info.icm_profile, xsane.scanner_default_gray_icm_profile, sizeof(image_info.icm_profile));
+        }
+      }
+      else
+      {
+        if (xsane.scanner_default_color_icm_profile)
+        {
+          strncpy(image_info.icm_profile, xsane.scanner_default_color_icm_profile, sizeof(image_info.icm_profile));
+        }
+      }
+
+
       xsane_write_pnm_header(xsane.out, &image_info, 0);
     }
 
@@ -1111,7 +1137,7 @@ void xsane_scan_done(SANE_Status status)
         xsane_read_pnm_header(infile, &image_info);
 
         /* open progressbar */
-        xsane_progress_new(PROGRESS_ROTATING_DATA, PROGRESS_TRANSFERING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
+        xsane_progress_new(PROGRESS_ROTATING_DATA, PROGRESS_TRANSFERRING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
         while (gtk_events_pending())
         {
           gtk_main_iteration();
@@ -1223,7 +1249,7 @@ void xsane_scan_done(SANE_Status status)
       { /* ok, we have to do a transformation */
 
         /* open progressbar */
-        xsane_progress_new(PROGRESS_CONVERTING_DATA, PROGRESS_TRANSFERING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
+        xsane_progress_new(PROGRESS_CONVERTING_DATA, PROGRESS_TRANSFERRING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
         while (gtk_events_pending())
         {
           gtk_main_iteration();
@@ -1233,12 +1259,12 @@ void xsane_scan_done(SANE_Status status)
 #ifdef HAVE_ANY_GIMP
         if (xsane.mode == XSANE_GIMP_EXTENSION)	/* xsane runs as gimp plugin */
         {
-          xsane_transfer_to_gimp(xsane.dummy_filename, xsane.progress_bar, &xsane.cancel_save);
+          xsane_transfer_to_gimp(xsane.dummy_filename, xsane.enable_color_management, preferences.cms_function, xsane.progress_bar, &xsane.cancel_save);
         }
         else
 #endif /* HAVE_ANY_GIMP */
         {
-          xsane_save_image_as(xsane.output_filename, xsane.dummy_filename, xsane.xsane_output_format, xsane.progress_bar, &xsane.cancel_save);
+          xsane_save_image_as(xsane.output_filename, xsane.dummy_filename, xsane.xsane_output_format, xsane.enable_color_management, preferences.cms_function, preferences.cms_intent, preferences.cms_bpc, xsane.progress_bar, &xsane.cancel_save);
         }
 
         xsane_progress_clear();
@@ -1282,7 +1308,7 @@ void xsane_scan_done(SANE_Status status)
       }
 
       /* open progressbar */
-      xsane_progress_new(PROGRESS_CONVERTING_DATA, PROGRESS_TRANSFERING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
+      xsane_progress_new(PROGRESS_CONVERTING_DATA, PROGRESS_TRANSFERRING_DATA, (GtkSignalFunc) xsane_cancel_save, &xsane.cancel_save);
 
       while (gtk_events_pending())
       {
@@ -1342,10 +1368,14 @@ void xsane_scan_done(SANE_Status status)
                       imagewidth, imageheight,
                       preferences.printer[preferences.printernr]->leftoffset   * 72.0/MM_PER_INCH, /* paper_left_margin */
                       preferences.printer[preferences.printernr]->bottomoffset * 72.0/MM_PER_INCH, /* paper_bottom_margin */
-                      preferences.printer[preferences.printernr]->width  * 72.0/MM_PER_INCH, /* usable paperwidth */
-                      preferences.printer[preferences.printernr]->height * 72.0/MM_PER_INCH, /* usable paperheight */
+                      preferences.printer[preferences.printernr]->width  * 72.0/MM_PER_INCH, /* usable paper_width */
+                      preferences.printer[preferences.printernr]->height * 72.0/MM_PER_INCH, /* usable paper_height */
                       preferences.paper_orientation,
-                      preferences.printer[preferences.printernr]->ps_flatdecoded, /* ps level 3 */
+                      preferences.printer[preferences.printernr]->ps_flatedecoded, /* ps level 3 */
+                      NULL /* hTransform */, xsane.enable_color_management,
+                      preferences.printer[preferences.printernr]->embed_csa, xsane.scanner_default_color_icm_profile,
+                      preferences.printer[preferences.printernr]->embed_crd, preferences.printer[preferences.printernr]->icm_profile, preferences.printer[preferences.printernr]->cms_bpc,
+                      0 /* intent */,
                       xsane.progress_bar,
                       &xsane.cancel_save);
       }
@@ -1444,7 +1474,7 @@ void xsane_scan_done(SANE_Status status)
       xsane_multipage_project_save();
 
       gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.multipage_status));
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
+      xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
     }
     else if (xsane.xsane_mode == XSANE_FAX)
     {
@@ -1485,7 +1515,7 @@ void xsane_scan_done(SANE_Status status)
       xsane_fax_project_save();
 
       gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.fax_status));
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
+      xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
     }
 #ifdef XSANE_ACTIVATE_EMAIL
     else if (xsane.xsane_mode == XSANE_EMAIL)
@@ -1527,7 +1557,7 @@ void xsane_scan_done(SANE_Status status)
       xsane_email_project_save();
 
       gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.email_status));
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
+      xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
     }
 #endif
   }
@@ -1568,7 +1598,7 @@ void xsane_scan_done(SANE_Status status)
     DBG(DBG_info, "Batch mode end of scan\n");
     sane_cancel(xsane.dev); /* we have to call sane_cancel otherwise we are not able to set new parameters */
   }
-//  else if ( ( (status != SANE_STATUS_GOOD) && (status != SANE_STATUS_EOF) ) || (!xsane.batch_loop) ) /* last scan: update histogram */
+/*  else if ( ( (status != SANE_STATUS_GOOD) && (status != SANE_STATUS_EOF) ) || (!xsane.batch_loop) ) */ /* last scan: update histogram */
   else
   {
     DBG(DBG_info, "Normal end of scan\n");
@@ -1729,7 +1759,7 @@ static void xsane_start_scan(void)
     image_info.image_width      = xsane.param.pixels_per_line;
     image_info.image_height     = xsane.param.lines;
     image_info.depth            = xsane.depth;
-    image_info.colors           = xsane.xsane_colors;
+    image_info.channels         = xsane.xsane_channels;
 
     image_info.resolution_x     = xsane.resolution_x;
     image_info.resolution_y     = xsane.resolution_y;
@@ -1753,6 +1783,29 @@ static void xsane_start_scan(void)
 
     image_info.reduce_to_lineart = xsane.expand_lineart_to_grayscale;
 
+    image_info.enable_color_management = xsane.enable_color_management;
+    image_info.cms_function            = preferences.cms_function;
+    image_info.cms_intent              = preferences.cms_intent;
+    image_info.cms_bpc                 = preferences.cms_bpc;
+
+
+    image_info.icm_profile[0] = 0; /* empty string */
+
+    if (image_info.channels == 1)
+    {
+      if (xsane.scanner_default_gray_icm_profile)
+      {
+        strncpy(image_info.icm_profile, xsane.scanner_default_gray_icm_profile, sizeof(image_info.icm_profile));
+      }
+    }
+    else
+    {
+      if (xsane.scanner_default_color_icm_profile)
+      {
+        strncpy(image_info.icm_profile, xsane.scanner_default_color_icm_profile, sizeof(image_info.icm_profile));
+      }
+    }
+
     xsane_write_pnm_header(xsane.out, &image_info, 0);
 
     fflush(xsane.out);
@@ -1946,34 +1999,6 @@ gint xsane_scan_dialog(gpointer *data)
       free(extension);
     }
   }
-#ifdef HAVE_ANY_GIMP
-  else	/* We are running in gimp mode */
-  {
-    if ((xsane.param.depth != 1) && (xsane.param.depth != 8)) /* not support bit depth ? */
-    {
-      if (!xsane.reduce_16bit_to_8bit) /* ask if reduce 16 to 8 bit */
-      {
-        if (xsane.param.depth == 16)
-        {
-          snprintf(buf, sizeof(buf), TEXT_GIMP_REDUCE_16BIT_TO_8BIT);
-          if (xsane_back_gtk_decision(ERR_HEADER_INFO, (gchar **) info_xpm, buf, BUTTON_REDUCE, BUTTON_CANCEL, TRUE /* wait */) == FALSE)
-          {
-            xsane_set_sensitivity(TRUE);
-           return FALSE;
-          }
-          xsane.reduce_16bit_to_8bit = TRUE;
-        }
-        else /* unsupported bit depth */
-        {
-          snprintf(buf, sizeof(buf), ERR_GIMP_BAD_DEPTH, xsane.param.depth);
-          xsane_back_gtk_error(buf, TRUE);
-          xsane_set_sensitivity(TRUE);
-         return FALSE;
-        }
-      }
-    }
-  }
-#endif
 
   if (xsane.dummy_filename) /* no dummy filename defined - necessary if an error occurs */
   {
@@ -1982,7 +2007,7 @@ gint xsane_scan_dialog(gpointer *data)
   }
 
   /* create scanner gamma tables, xsane internal gamma tables are created after sane_start */
-  if ( (xsane.xsane_colors > 1) && /* color scan */
+  if ( (xsane.xsane_channels > 1) && /* color scan */
        xsane.scanner_gamma_color ) /* gamma table for red, green and blue available */
   {
    double gamma_red, gamma_green, gamma_blue;
@@ -2097,7 +2122,7 @@ gint xsane_scan_dialog(gpointer *data)
 
     xsane.gamma_data = malloc(gamma_gray_size * sizeof(SANE_Int));
 
-    if (xsane.xsane_colors > 1) /* color scan */
+    if (xsane.xsane_channels > 1) /* color scan */
     {
       xsane_create_gamma_curve(xsane.gamma_data, xsane.negative,
                                gamma, xsane.brightness, xsane.contrast,
@@ -2145,7 +2170,7 @@ static void xsane_create_internal_gamma_tables(void)
   size = (int) pow(2, xsane.param.depth);
   maxval = size-1;
 
-  if (xsane.xsane_colors > 1) /* color scan */
+  if (xsane.xsane_channels > 1) /* color scan */
   {
     if ( (!xsane.scanner_gamma_color) && (xsane.scanner_gamma_gray) )
     {
