@@ -71,6 +71,8 @@
 
 #ifdef HAVE_LIBLCMS
 # include "lcms.h"
+#else
+# define cmsHTRANSFORM void *
 #endif
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -89,7 +91,7 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-#define XSANE_VERSION		"0.992"
+#define XSANE_VERSION		"0.995"
 #define XSANE_AUTHOR		"Oliver Rauch"
 #define XSANE_COPYRIGHT		"Oliver Rauch"
 #define XSANE_DATE		"1998-2007"
@@ -101,6 +103,7 @@
 
 #define XSANE_DEBUG_ENVIRONMENT	"XSANE_DEBUG"
 
+#define XSANE_PROGRESS_BAR_MIN_DELTA_PERCENT 0.025
 #define XSANE_DEFAULT_UMASK		0007
 #define XSANE_HOLD_TIME			200
 #define XSANE_CONTINUOUS_HOLD_TIME	10
@@ -303,6 +306,12 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+#ifndef PATH_MAX
+# define PATH_MAX	1024
+#endif
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 enum
 {
   XSANE_PATH_LOCAL_SANE = 0,
@@ -385,7 +394,7 @@ typedef struct Image_info
     int image_height;
 
     int depth;
-    int colors;
+    int channels;
 
     double resolution_x;
     double resolution_y;
@@ -408,6 +417,13 @@ typedef struct Image_info
     double threshold;
 
     int reduce_to_lineart;
+
+    int enable_color_management;
+    int cms_function;
+    int cms_intent;
+    int cms_bpc;
+
+    char icm_profile[PATH_MAX];
   }
 Image_info;
 
@@ -473,19 +489,20 @@ enum
   EMAIL_AUTH_ASMTP_LOGIN,
   EMAIL_AUTH_ASMTP_CRAM_MD5
 };
-                                                                                                                 
+
+enum
+{
+  XSANE_CMS_FUNCTION_EMBED_SCANNER_ICM_PROFILE = 0,
+  XSANE_CMS_FUNCTION_CONVERT_TO_SRGB,
+  XSANE_CMS_FUNCTION_CONVERT_TO_WORKING_CS
+};
+
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
 extern void xsane_pref_save(void);
 extern void xsane_interface(int argc, char **argv);
 extern void xsane_batch_scan_add(void);
 
-/* ---------------------------------------------------------------------------------------------------------------------- */
-
-#ifndef PATH_MAX
-# define PATH_MAX	1024
-#endif
-
 /* ---------------------------------------------------------------------------------------------------------------------- */
 #ifndef TEMP_PATH
 # define TEMP_PATH		/tmp/
@@ -744,6 +761,9 @@ typedef struct Xsane
     GtkWidget *filename_entry;
     GtkWidget *filetype_option_menu;
 
+    /* for all modes */
+    GtkWidget *cms_function_option_menu;
+
     /* saving and transformation values: */
     FILE *out;
     int xsane_mode;
@@ -838,7 +858,7 @@ typedef struct Xsane
     GtkWidget *contrast_blue_widget;
     GtkWidget *threshold_widget;
 
-    SANE_Int xsane_colors;
+    SANE_Int xsane_channels;
     SANE_Bool scanner_gamma_color;
     SANE_Bool scanner_gamma_gray;
 
@@ -954,9 +974,9 @@ typedef struct Xsane
     int show_preview;
 
     int enable_color_management; 
-    int embed_icm_profile;
-    char *scanner_refl_icm_profile;
-    char *scanner_tran_icm_profile;
+    char *scanner_active_icm_profile;
+    char *scanner_default_color_icm_profile;
+    char *scanner_default_gray_icm_profile;
 
     int print_filenames;
     int force_filename;
@@ -989,9 +1009,12 @@ typedef struct XsaneSetup
   GtkWidget *printer_gamma_green_entry;
   GtkWidget *printer_gamma_blue_entry;
   GtkWidget *printer_icm_profile_entry;
+  GtkWidget *printer_embed_csa_button;
+  GtkWidget *printer_embed_crd_button;
+  GtkWidget *printer_cms_bpc_button;
   GtkWidget *printer_width_entry;
   GtkWidget *printer_height_entry;
-  GtkWidget *printer_ps_flatdecoded_button;
+  GtkWidget *printer_ps_flatedecoded_button;
 
   GtkWidget *jpeg_image_quality_scale;
   GtkWidget *png_image_compression_scale;
@@ -1000,8 +1023,8 @@ typedef struct XsaneSetup
   GtkWidget *overwrite_warning_button;
   GtkWidget *increase_filename_counter_button;
   GtkWidget *skip_existing_numbers_button;
-  GtkWidget *save_ps_flatdecoded_button;
-  GtkWidget *save_pdf_flatdecoded_button;
+  GtkWidget *save_ps_flatedecoded_button;
+  GtkWidget *save_pdf_flatedecoded_button;
   GtkWidget *save_pnm16_as_ascii_button;
   GtkWidget *reduce_16bit_to_8bit_button;
 
@@ -1018,7 +1041,7 @@ typedef struct XsaneSetup
   GtkWidget *preview_threshold_mul_entry;
   GtkWidget *preview_threshold_off_entry;
   GtkWidget *auto_enhance_gamma_button;
-  GtkWidget *preselect_scanarea_button;
+  GtkWidget *preselect_scan_area_button;
   GtkWidget *auto_correct_colors_button;
   GtkWidget *disable_gimp_preview_gamma_button;
   GtkWidget *preview_oversampling_entry;
@@ -1035,7 +1058,7 @@ typedef struct XsaneSetup
   GtkWidget *fax_leftoffset_entry;
   GtkWidget *fax_bottomoffset_entry;
   GtkWidget *fax_height_entry;
-  GtkWidget *fax_ps_flatdecoded_button;
+  GtkWidget *fax_ps_flatedecoded_button;
 
   GtkWidget *tmp_path_entry;
 
@@ -1056,11 +1079,14 @@ typedef struct XsaneSetup
   GtkWidget *ocr_gui_outfd_option_entry;
   GtkWidget *ocr_progress_keyword_entry;
 
-  GtkWidget *embed_icm_profile_button;
-  GtkWidget *scanner_refl_icm_profile_entry;
-  GtkWidget *scanner_tran_icm_profile_entry;
+  GtkWidget *cms_intent_option_menu;
+  GtkWidget *cms_bpc_button;
+  GtkWidget *embed_scanner_icm_profile_for_gimp_button;
+  GtkWidget *scanner_default_color_icm_profile_entry;
+  GtkWidget *scanner_default_gray_icm_profile_entry;
   GtkWidget *display_icm_profile_entry;
   GtkWidget *custom_proofing_icm_profile_entry;
+  GtkWidget *working_color_space_icm_profile_entry;
 
   int filename_counter_len;
 
